#include "ktnefview.h"
#include "ktnefattach.h"

#include <qheader.h>
#include <qpixmap.h>
#include <kapp.h>
#include <kiconloader.h>

#include <kmimetype.h>

class Attachment : public QListViewItem
{
public:
	Attachment(QListView *parent, KTNEFAttach *attach);
	~Attachment();

	KTNEFAttach* getAttachment() const { return attach_; }

private:
	KTNEFAttach	*attach_;
};

Attachment::Attachment(QListView *parent, KTNEFAttach *attach)
	: QListViewItem(parent, attach->name().data()), attach_(attach)
{
	QString	num;
	num.setNum(attach_->size());
	setText(2, num.data());
	if (!attach_->fileName().isEmpty()) setText(0, attach_->fileName());
	KMimeType::Ptr	mimeType;
	if (attach_->mimeTag().isEmpty())
	{
		mimeType = KMimeType::findByURL(KURL("file:/"+(attach_->fileName().isEmpty() ? attach_->name() : attach_->fileName())));
		// no mime type was given in the TNEF file, set it to the one found here
		attach->setMimeTag(mimeType->name());
	}
	else
		mimeType = KMimeType::mimeType(attach_->mimeTag());
	setText(1, mimeType->comment());
	setPixmap(0, mimeType->pixmap(KIcon::Small));
}

Attachment::~Attachment()
{
}

//------------------------------------------------------------------------------------------------------

KTNEFView::KTNEFView(QWidget *parent, const char *name)
	: QListView(parent,name)
{
	attachments_.setAutoDelete(false);
	addColumn("File name");
	addColumn("File type");
	addColumn("Size");
	setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	setLineWidth(1);
	setSelectionMode(QListView::Extended);
	setHScrollBarMode(QScrollView::AlwaysOff);
	setVScrollBarMode(QScrollView::AlwaysOn);
}

KTNEFView::~KTNEFView()
{
}

void KTNEFView::setAttachments(QList<KTNEFAttach> *list)
{
	clear();
	if (list)
	{
		QListIterator<KTNEFAttach>	it(*list);
		for (;it.current();++it)
			new Attachment(this, it.current());
	}
}

void KTNEFView::resizeEvent(QResizeEvent *e)
{
	int	w = visibleWidth()/2;
	setColumnWidth(0,w);
	setColumnWidth(1,w/2);
	setColumnWidth(2,w/2);
	resizeContents(visibleWidth(),visibleHeight());
	if (e) QListView::resizeEvent(e);
}

QList<KTNEFAttach>* KTNEFView::getSelection()
{
	attachments_.clear();
	QListViewItem	*item = firstChild();
	while (item)
	{
		if (item->isSelected()) attachments_.append(((Attachment*)item)->getAttachment());
		item = item->nextSibling();
	}
	return &attachments_;
}
